//*******************************************************
//	NS73M FM transmitter controler
//
//	MCU:	ATtiny13
//			hfuse:FBh	BOD 2.7V
//			lfuse:22h	0.6MHz
//
//	FM transmitter module : NS73M (NIIGATA SEIMITSU CO., LTD.)
//
//	Compiler:	AVR-GCC		3.4.5
//				AVR-LIBC	1.4.2
//				binutils	2.16.1
//
//	2008.8.8	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>

#define		NOP		asm volatile("nop\n"::)
#define		WDR		asm volatile("wdr\n"::)
#define		SLEEP	asm volatile("sleep\n"::)

typedef	unsigned char	uchar;
typedef	unsigned int	uint;
typedef	unsigned long	ulong;

#define		EEPROM_FREQ	(uint16_t *)1

// ݡ(PB)
#define		DA		0	// PB
#define		LED		1
#define		CK		2
#define		NC		3
#define		LA		4

void set_data(uchar reg, uchar data);
void wait_100ms(uchar n);


//*******************************************************
//	ᥤ롼
//*******************************************************
int main (void) {
	ulong	freq;
	uchar	mod;
	uchar	cex;

	cli();	// ߶ػ

	// ݡ
	PORTB = (1<<NC);							// ץ륢å
	DDRB = (1<<LA)|(1<<LED)|(1<<DA)|(1<<CK);	// ϥݡ

	// ꡼ư
	MCUCR = (1<<SE)|(1<<SM1);	// ꡼׵, ѥ⡼

	// 0
	TCCR0A = 0;
	TCCR0B = (1<<CS02)|(1<<CS00);	// 0.6MHz/1024=586Hz

	// 0.5ûԤ
	wait_100ms(5);

	// EEPROMȿɤ߹
	freq = (ulong)eeprom_read_word(EEPROM_FREQ);
	if ((freq < 7600)||(freq > 10800)){
		freq = 8820;	// ǥե 88.2MHz
	}

	// CEX
	if (freq < 9000){
		cex = 3;
	} else if (freq < 9600){
		cex = 2;
	} else if (freq < 10200){
		cex = 1;
	} else {
		cex = 0;
	}

	// 쥸ͷ׻
	freq *= 10000;
	freq += 304000;
	freq /= (8192 / 2);
	mod = (uchar)freq & 1;	// Σ
	freq /= 2;
	freq += mod;
	
	set_data(0x0E, 0x05);	// ꥻå
	set_data(0x01, 0xB4);	// Pilot ON, Forced subcarrier
	set_data(0x02, 0x05);	// ULD ON, 0.5mW
	set_data(0x03, (uchar)freq);		// ȿ
	set_data(0x04, (uchar)(freq >> 8));	// ȿ
	set_data(0x08, 0x18 + cex);	// Local OSC Band
	//set_data(0x00, 0xA1);	// Audio input 200mV, Pre-emphasis 75us, Power ON
	set_data(0x00, 0x01);	// Audio input 100mV, Pre-emphasis 50us, Power ON

	set_data(0x0E, 0x05);	// ꥻå
	set_data(0x06, 0x1E);	// Charge pump

	PORTB |= (1<<LED);		// LED

	SLEEP;	// Ͽ

	for (;;){
	}
}


//*******************************************************
//	100msñ̤ǻԤ
//*******************************************************
void wait_100ms(uchar n){

	while(n > 0){
		// 0.1ûԤ
		TCNT0 = 256 - 59;
		TIFR0 |= (1<<TOV0);	// Сեե饰ꥢ
		while(!(TIFR0 & (1<<TOV0)));
		--n;
	}
}


//*******************************************************
//	NS73M 쥸˥ǡå
//*******************************************************
void set_data(uchar adrs, uchar data){
	uint	i;

	for (i = 0; i < 4; i++){
		if (adrs & 1){
			PORTB |= (1<<DA);
		} else {
			PORTB &= ~(1<<DA);
		}
		PORTB |= (1<<CK);
		NOP;
		PORTB &= ~(1<<CK);
		adrs >>= 1;
	}

	for (i = 0; i < 8; i++){
		if (data & 1){
			PORTB |= (1<<DA);
		} else {
			PORTB &= ~(1<<DA);
		}
		PORTB |= (1<<CK);
		NOP;
		PORTB &= ~(1<<CK);
		data >>= 1;
	}

	PORTB |= (1<<LA);
	NOP;
	PORTB &= ~(1<<LA);
}


